/* Copyright (c) 1997, MpegTV LLC */
#ifndef _pmp_h
#define _pmp_h
#ifndef lint
static char rcsid_pmp_h[] = "$Header: /net/hopkins/var/home/tristan/mpeg/dec/RCS/pmp.h,v 1.26 1997/08/21 06:42:56 tristan Exp $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "misc.h"

#if 0 /* Tristan Savatier - Fri Aug 15 02:40:26 PDT 1997 */
//#include <sys/ipc.h>
//#include <sys/msg.h>
//#include <sys/param.h>
//#include <sys/types.h>
#endif /* 0 - Tristan Savatier - Fri Aug 15 02:40:26 PDT 1997 */

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/

/* indexes for the pipes in PMP_CHANNEL */
typedef enum {
	PMP_FD_READ = 0,
	PMP_FD_WRITE = 1,
	PMP_FD_COUNT = 2	/* not an index! this is the number of indexes */
} PMP_FD;

/*
 * Communication channel.
 * The ack pipe is used for synchronous communications, i.e.
 * to acknowledge that a message was received.
 */
typedef struct {
	int		msg_pipe[PMP_FD_COUNT];
	int		ack_pipe[PMP_FD_COUNT];
	u_int	serial_number;	/* serial number of the next command to be sent */
} PMP_CHANNEL;

 
/*
 * Communication interface with Player.
 *
 * Commands to the player are sent on the send_channel.
 * Messages from the player are received on the receive_channel.
 *
 * Command result message (i.e. success/failure and error codes), when
 * requested, are also received (asynchronously) on the receive_channel.
 * Command results include a copy on the original command message header.
 * This header contains the serial number of the original command.
 * This serial number can be used to match a Command result message
 * with corresponding command.
 */
typedef struct {
	pid_t			pid;				/* player's pid */
	PMP_CHANNEL		send_channel;
	PMP_CHANNEL		receive_channel;
} PMP_STRUCT;


typedef struct {
	int16		serial_number;
	int16		flags;
#define		PMP_FLAG_ACK	(1 << 0)	/* synchronous mode (obsolete) */

/* specify parameter type */
#define		PMP_FLAG_TYPE_CUSTOM	(0 << 1)
#define		PMP_FLAG_TYPE_VOID		(1 << 1)
#define		PMP_FLAG_TYPE_INT		(2 << 1)
#define		PMP_FLAG_TYPE_DOUBLE	(3 << 1)
#define		PMP_FLAG_TYPE_STRING	(4 << 1)
#define		PMP_FLAG_TYPE_RESERVED1	(5 << 1)
#define		PMP_FLAG_TYPE_RESERVED2	(6 << 1)
#define		PMP_FLAG_TYPE_RESERVED3	(7 << 1)

#define		PMP_FLAG_TYPE_MASK		(7 << 1)
#define		PMP_FLAG_TYPE(msg)		((msg)->flags & PMP_FLAG_TYPE_MASK)

#define		PMP_FLAG_RETURN_STATUS		(1 << 4)	/* request command result */
#define		PMP_FLAG_RETURN_ORIGINAL	(1 << 5)	/* copy original message after result */

	u_int16		type;
#define		PMP_TYPE_ACK	((u_int16)(-1))

	u_int16		size;
} PMP_HEADER;

/* standard message data */
typedef union {
	int		data_int;
	double	data_double;
	char	data_string[PATH_MAX];
	char	data_reserved[1024];	/* see note below */ 
} PMP_DATA_GENERIC;

/*
 * data_reserved is added only to make sure that PMP_DATA_GENERIC is "large enough".
 * The idea is to secretely reserve enough room in unions that contain PMP_DATA_GENERIC,
 * to take care of forward compatible extension.
 * For example, this way, old applications that PMP_DATA_A2G to read messages will
 * not crash if a newer mtvp sends a longer message (longer than any of the
 * types in th old PMP_DATA_A2G union).
 */

/*--------------------------------------------------------------------------+
| Externals																	|
+--------------------------------------------------------------------------*/


EXTERN_FUNCTION(int pmp_receive_msg, (const PMP_CHANNEL *, PMP_HEADER *, char *, u_int));
EXTERN_FUNCTION(int pmp_send_msg, (PMP_CHANNEL *, PMP_HEADER *, const char *));
EXTERN_FUNCTION(int pmp_send_ack, (const PMP_CHANNEL *, PMP_HEADER *));
EXTERN_FUNCTION(Bool pmp_check, (const PMP_CHANNEL *));
EXTERN_FUNCTION(int pmp_init_queue, (PMP_CHANNEL *));
EXTERN_FUNCTION(int pmp_close_read, (PMP_CHANNEL *));
EXTERN_FUNCTION(int pmp_close_write, (PMP_CHANNEL *));
EXTERN_FUNCTION(const char * pmp_print_msg, (const PMP_HEADER *, const char *));

#endif /* _pmp_h */
